/*
 Copyright 2006 Opera Software ASA. All rights reserved.
 
This file contains Original Code and/or Contributions to the Original 
Code as defined in the Opera Web Applications License (the License). 
You may not use this file except in compliance with the License. Please
obtain a copy of the License at http://www.opera.com/license/owal/
and read it before using this file. 

The Original Code and all Contributions to the Original Code distributed 
under the License are distributed on an AS IS basis, WITHOUT WARRANTY
OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND OPERA HEREBY DISCLAIMS ALL 
SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF 
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT. 

Please see the License for the specific language governing rights and 
limitations under the License.  */

var Clock = function(width,height){
  var self=this;
  this.width = width;
  this.height = height;
  this.element = document.createElement('canvas');
  this.element.setAttribute('width',this.width);
  this.element.setAttribute('height',this.height);
  this.onbeforetick = null;
  this.onaftertick = null;

  this.canvas = this.element.getContext('2d');

  var time;
  var Tick = function(){
    var date = new Date();
    this.hours = date.getHours();
    this.minutes = date.getMinutes();
    this.seconds = date.getSeconds();
    this.mseconds = date.getMilliseconds();
  }
  
  this.features = {
    hours   : true,
    minutes : true,
    seconds : true,
    shadows : true,
    smooth  : false
  }
  
  var Hand = function(){
    var super = this;
    this.color = "#fff";
    this.width  = 4;
    this.stroke_width = 3;
    this.stroke_color = "black";
    this.length = self.width/2;
    this.overlap = 8;
    this.shadow_offsetX = 1;
    this.shadow_offsetY = 1;
    this.draw = function(){

      if (self.features.shadows){
        self.canvas.save();
        self.canvas.beginPath();
        self.canvas.translate(self.width/2+super.shadow_offsetX,self.height/2+super.shadow_offsetY);
        self.canvas.rotate(super.getAngle());
        self.canvas.translate(-super.width/2,-super.overlap)
        self.canvas.fillStyle = "rgba(0,0,0,0.75)";
        self.canvas.fillRect(0,0,super.width,super.length);
        self.canvas.closePath();
        self.canvas.restore();    
      }
      
      self.canvas.save();
      self.canvas.beginPath();
      self.canvas.translate(self.width/2,self.height/2);
      self.canvas.rotate(super.getAngle());
      self.canvas.translate(-super.width/2,-super.overlap)
      self.canvas.fillStyle = super.color;
      self.canvas.strokeStyle = super.stroke_color;
      self.canvas.strokeWidth = super.stroke_width;
      self.canvas.fillRect(0,0,super.width,super.length); 
      if (super.stroke_width) self.canvas.strokeRect(0,0,super.width,super.length); 
      self.canvas.closePath();
      self.canvas.restore();
    }
    
    // clock hands need to override this
    this.getAngle = function(){return 0;}
    
  }
  
  this.hour = new Hand();
  this.hour.stroke_style = "rgba(0,0,0,0.75)";
  this.hour.length = this.width/3;
  this.hour.getAngle = function(){ 
    return ((time.hours%12)/6 + time.minutes/360 + time.seconds/21600)*Math.PI+Math.PI;
  }
    
  this.minute = new Hand();
  this.minute.stroke_style = "rgba(0,0,0,0.75)";

  this.minute.getAngle = function(){
    return (time.minutes/30 + time.seconds/1800)*Math.PI+Math.PI;
  }

  this.second = new Hand();
  this.second.width = 2;
  this.second.color = "#b00";
  this.second.stroke_width = 2;
  this.second.stroke_style = "rgba(0,0,0,0.75)";
  this.second.getAngle = function(){
    if (self.features.smooth) 
      return (time.seconds/30 + time.mseconds/30000)*Math.PI+Math.PI;
    else 
      return (time.seconds/30)*Math.PI+Math.PI;
  }

  var onbeforeticks = [];
  var onafterticks = [];

  var draw = function(){
    time = new Tick();
    self.canvas.clearRect(0,0,self.width,self.height);
    for (var i = 0, callback; callback = onbeforeticks[i];i++){
      callback.apply(self);
    }
    self.canvas.save();
    if (self.features.hours) self.hour.draw();
    if (self.features.minutes) self.minute.draw();
    if (self.features.seconds) self.second.draw();
    self.canvas.restore();    
    for (var i = 0, callback; callback = onafterticks[i];i++){
      callback.apply(self);
    }
  }
  
  var timer = null;
  
  this.start = function(){
    draw();
    timer = setInterval(draw, this.features.smooth?54:1000);
  }
  
  this.stop = function(){
    clearInterval(timer);
  }
  
  this.addCallback = function(type,callback){
    switch (type){
      case "beforetick":
        onbeforeticks[onbeforeticks.length] = callback;
        break;
      case "aftertick":
        onafterticks[onafterticks.length] = callback;
        break;
      default:
        break;
    }
  }
}
